"use client"

import { createClient } from "@/utils/supabase"
import { useState, useEffect, useCallback, useRef } from "react"
import { useAuth, useTVMode } from "@/components/context"
import { getRecentUsersWithStatuses, updateStatuses, updateUserStatus, type UserWithStatus } from "@/lib/hooks"
import { BasedAvatar, Drawer, DrawerTrigger, Loading } from "@/components/ui"
import { SubmitButton } from "@/components/default"
import { toast } from "sonner"
import { HistoryDrawer } from "@/components/status"
import type { Profile } from "@/utils/supabase"
import type { RealtimeChannel } from "@supabase/supabase-js"
import { makeConditionalClassName } from "@/lib/utils"
import { Card, CardContent, CardHeader } from "@/components/ui/card"
import { Badge } from "@/components/ui/badge"
import { Checkbox } from "@/components/ui/checkbox"
import { Input } from "@/components/ui/input"
import { Button } from "@/components/ui/button"
import { Wifi, WifiOff, Clock, User } from "lucide-react"

type StatusListProps = {
  initialStatuses: UserWithStatus[]
}

export const StatusList = ({ initialStatuses = [] }: StatusListProps) => {
  const { isAuthenticated } = useAuth()
  const { tvMode } = useTVMode()
  const [loading, setLoading] = useState(true)
  const [selectedUsers, setSelectedUsers] = useState<string[]>([])
  const [selectAll, setSelectAll] = useState(false)
  const [updatingStatus, setUpdatingStatus] = useState(false)
  const [statusInput, setStatusInput] = useState("")
  const [usersWithStatuses, setUsersWithStatuses] = useState<UserWithStatus[]>(initialStatuses)
  const [selectedHistoryUser, setSelectedHistoryUser] = useState<Profile | null>(null)
  const [connectionStatus, setConnectionStatus] = useState<"connecting" | "connected" | "disconnected">("connecting")
  const [newStatusIds, setNewStatusIds] = useState<Set<string>>(new Set())
  const channelRef = useRef<RealtimeChannel | null>(null)
  const supabaseRef = useRef(createClient())

  const fetchRecentUsersWithStatuses = useCallback(async () => {
    try {
      const response = await getRecentUsersWithStatuses()
      if (!response.success) throw new Error(response.error)
      return response.data
    } catch (error) {
      toast.error(`Error fetching technicians: ${error as Error}`)
      return []
    }
  }, [])

  // Initial load
  useEffect(() => {
    const loadData = async () => {
      const data = await fetchRecentUsersWithStatuses()
      setUsersWithStatuses(data)
      setLoading(false)
    }
    loadData().catch((error) => {
      console.error("Error loading data:", error)
    })
  }, [fetchRecentUsersWithStatuses, isAuthenticated])

  const updateStatus = useCallback(async () => {
    if (!isAuthenticated) {
      toast.error("You must be signed in to update statuses.")
      return
    }
    if (!statusInput.trim()) {
      toast.error("Please enter a valid status.")
      return
    }

    try {
      setUpdatingStatus(true)
      if (selectedUsers.length === 0) {
        const result = await updateUserStatus(statusInput)
        if (!result.success) throw new Error(result.error)
        toast.success("Status updated for signed in user.")
      } else {
        const selectedUserObjects = usersWithStatuses.filter((u) => selectedUsers.includes(u.user.id))
        const result = await updateStatuses(selectedUserObjects, statusInput)
        if (!result.success) throw new Error(result.error)
        toast.success(`Status updated for ${selectedUsers.length} selected users.`)
      }
      setSelectedUsers([])
      setStatusInput("")
    } catch (error) {
      const errorMessage = error instanceof Error ? error.message : String(error)
      toast.error(`Failed to update status: ${errorMessage}`)
    } finally {
      setUpdatingStatus(false)
    }
  }, [isAuthenticated, statusInput, selectedUsers, usersWithStatuses])

  const handleCheckboxChange = (id: string) => {
    setSelectedUsers((prev) => (prev.includes(id) ? prev.filter((prevId) => prevId !== id) : [...prev, id]))
  }

  const handleSelectAllChange = () => {
    if (selectAll) {
      setSelectedUsers([])
    } else {
      setSelectedUsers(usersWithStatuses.map((user) => user.user.id))
    }
    setSelectAll(!selectAll)
  }

  useEffect(() => {
    setSelectAll(selectedUsers.length === usersWithStatuses.length && usersWithStatuses.length > 0)
  }, [selectedUsers.length, usersWithStatuses.length])

  // Real-time connection setup
  useEffect(() => {
    if (!isAuthenticated) return

    let reconnectAttempts = 0
    const maxReconnectAttempts = 3
    let reconnectTimeout: NodeJS.Timeout
    let isComponentMounted = true
    let currentChannel: RealtimeChannel | null = null

    const setupRealtimeConnection = () => {
      if (!isComponentMounted) return

      // Clean up any existing channel first
      if (currentChannel) {
        supabaseRef.current.removeChannel(currentChannel).catch(console.error)
        currentChannel = null
      }

      console.log("Setting up new realtime connection...")
      setConnectionStatus("connecting")

      const channel = supabaseRef.current
        .channel(`status_updates`, {
          config: {
            broadcast: { self: true },
          },
        })
        .on("broadcast", { event: "status_updated" }, (payload) => {
          const { user_status } = payload.payload as {
            user_status: UserWithStatus
            timestamp: string
          }
          console.log("Received status update:", user_status)

          // Add animation class for new status
          setNewStatusIds((prev) => new Set([...prev, user_status.user.id]))

          // Remove animation class after animation completes
          setTimeout(() => {
            setNewStatusIds((prev) => {
              const newSet = new Set(prev)
              newSet.delete(user_status.user.id)
              return newSet
            })
          }, 1000)

          setUsersWithStatuses((prevUsers) => {
            const existingUserIndex = prevUsers.findIndex((u) => u.user.id === user_status.user.id)

            if (existingUserIndex !== -1) {
              const updatedUsers = [...prevUsers]
              updatedUsers[existingUserIndex] = {
                user: user_status.user,
                status: user_status.status,
                created_at: user_status.created_at,
                updated_by: user_status.updated_by,
              }
              return updatedUsers
            } else {
              return [user_status, ...prevUsers]
            }
          })
        })
        .subscribe((status) => {
          console.log("Subscription status:", status)
          // eslint-disable-next-line @typescript-eslint/no-unsafe-enum-comparison
          if (status === "SUBSCRIBED") {
            console.log("Successfully subscribed to status updates!")
            setConnectionStatus("connected")
            reconnectAttempts = 0
          // eslint-disable-next-line @typescript-eslint/no-unsafe-enum-comparison
          } else if (status === "CLOSED") {
            console.log("Connection closed")
            setConnectionStatus("disconnected")

            if (isComponentMounted && reconnectAttempts < maxReconnectAttempts) {
              reconnectAttempts++
              const delay = 2000 * reconnectAttempts

              console.log(`Reconnecting after close ${reconnectAttempts}/${maxReconnectAttempts} in ${delay}ms`)

              if (reconnectTimeout) {
                clearTimeout(reconnectTimeout)
              }

              reconnectTimeout = setTimeout(() => {
                if (isComponentMounted) {
                  setupRealtimeConnection()
                }
              }, delay)
            } else {
              console.log("Max reconnection attempts reached or component unmounted")
            }
          // eslint-disable-next-line @typescript-eslint/no-unsafe-enum-comparison
          } else if (status === "CHANNEL_ERROR") {
            console.error("Channel error - stopping reconnection attempts")
            setConnectionStatus("disconnected")
          }
        })

      currentChannel = channel
      channelRef.current = channel
    }

    const initialTimeout = setTimeout(() => {
      if (isComponentMounted) {
        setupRealtimeConnection()
      }
    }, 1000)

    return () => {
      isComponentMounted = false

      if (initialTimeout) {
        clearTimeout(initialTimeout)
      }

      if (reconnectTimeout) {
        clearTimeout(reconnectTimeout)
      }

      if (currentChannel) {
        console.log("Cleaning up realtime connection...")
        supabaseRef.current.removeChannel(currentChannel).catch((error) => {
          console.error(`Error unsubscribing: ${error}`)
        })
      }

      channelRef.current = null
    }
  }, [isAuthenticated])

  const formatTime = (timestamp: string) => {
    const date = new Date(timestamp)
    const time = date.toLocaleTimeString("en-US", {
      hour: "numeric",
      minute: "numeric",
    })
    const day = date.getDate()
    const month = date.toLocaleString("default", { month: "long" })
    return `${time} - ${month} ${day}`
  }

  const getConnectionIcon = () => {
    switch (connectionStatus) {
      case "connected":
        return <Wifi className="w-4 h-4 text-green-500" />
      case "connecting":
        return <Wifi className="w-4 h-4 text-yellow-500 animate-pulse" />
      case "disconnected":
        return <WifiOff className="w-4 h-4 text-red-500" />
    }
  }

  const getConnectionText = () => {
    switch (connectionStatus) {
      case "connected":
        return "Connected"
      case "connecting":
        return "Connecting..."
      case "disconnected":
        return "Disconnected"
    }
  }

  if (loading) {
    return (
      <div className="flex justify-center items-center min-h-[400px]">
        <Loading className="w-full" alpha={0.5} />
      </div>
    )
  }

  const containerClassName = makeConditionalClassName({
    context: tvMode,
    defaultClassName: "mx-auto space-y-4",
    on: "lg:w-11/12 w-full mt-15",
    off: "w-5/6",
  })

  const cardClassName = makeConditionalClassName({
    context: tvMode,
    defaultClassName: "transition-all duration-300 hover:shadow-md",
    on: "lg:text-4xl",
    off: "lg:text-base",
  })

  return (
    <div className={containerClassName}>
      {/* Connection Status Header */}
      <div className="flex items-center justify-between mb-6">
        <div className="flex items-center gap-2">
          <h2 className={`font-bold ${tvMode ? "text-6xl" : "text-2xl"}`}>Tech Status</h2>
          <Badge variant="outline" className="flex items-center gap-2">
            {getConnectionIcon()}
            <span className="text-xs">{getConnectionText()}</span>
          </Badge>
        </div>

        {!tvMode && usersWithStatuses.length > 0 && (
          <div className="flex items-center gap-2">
            <Checkbox id="select-all" checked={selectAll} onCheckedChange={handleSelectAllChange} />
            <label htmlFor="select-all" className="text-sm font-medium">
              Select All ({selectedUsers.length} selected)
            </label>
          </div>
        )}
      </div>

      {/* Status Cards */}
      <div className="space-y-3">
        {usersWithStatuses.map((userWithStatus) => {
          const isSelected = selectedUsers.includes(userWithStatus.user.id)
          const isNewStatus = newStatusIds.has(userWithStatus.user.id)
          const isUpdatedByOther = userWithStatus.updated_by && userWithStatus.updated_by.id !== userWithStatus.user.id

          return (
            <Card
              key={userWithStatus.user.id}
              className={`
                ${cardClassName}
                ${isSelected ? "ring-2 ring-primary" : ""}
                ${isNewStatus ? "animate-pulse bg-primary/5 border-primary/20" : ""}
                hover:bg-muted/50 cursor-pointer
              `}
            >
              <CardHeader className="pb-3">
                <div className="flex items-center justify-between">
                  <div className="flex items-center gap-3">
                    {!tvMode && (
                      <Checkbox
                        checked={isSelected}
                        onCheckedChange={() => handleCheckboxChange(userWithStatus.user.id)}
                        onClick={(e) => e.stopPropagation()}
                      />
                    )}
                    <BasedAvatar
                      src={userWithStatus.user.avatar_url}
                      fullName={userWithStatus.user.full_name}
                      className={tvMode ? "w-16 h-16" : "w-12 h-12"}
                    />
                    <div>
                      <h3 className={`font-semibold ${tvMode ? "text-5xl" : "text-lg"}`}>
                        {userWithStatus.user.full_name ?? "Unknown User"}
                      </h3>
                      {isUpdatedByOther && (
                        <div className="flex items-center gap-1 text-muted-foreground">
                          <BasedAvatar
                            src={userWithStatus.updated_by?.avatar_url}
                            fullName={userWithStatus.updated_by?.full_name}
                            className="w-3 h-3"
                          />
                          {userWithStatus.updated_by && (
                            <span className={`text-xs ${tvMode ? "text-3xl" : ""}`}>
                              Updated by {userWithStatus.updated_by.full_name}
                            </span>
                          )}
                        </div>
                      )}
                    </div>
                  </div>
                  <div className="flex items-center gap-2 text-muted-foreground">
                    <Clock className={`${tvMode ? "w-8 h-8" : "w-4 h-4"}`} />
                    <span className={`text-sm ${tvMode ? "text-3xl" : ""}`}>
                      {formatTime(userWithStatus.created_at)}
                    </span>
                  </div>
                </div>
              </CardHeader>
              <CardContent className="pt-0">
                <Drawer>
                  <DrawerTrigger asChild>
                    <div
                      className={`
                        p-4 rounded-lg bg-muted/30 hover:bg-muted/50
                        transition-colors cursor-pointer text-left
                        ${tvMode ? "text-4xl" : "text-base"}
                      `}
                      onClick={() => setSelectedHistoryUser(userWithStatus.user)}
                    >
                      <p className="font-medium">{userWithStatus.status}</p>
                    </div>
                  </DrawerTrigger>
                  {selectedHistoryUser === userWithStatus.user && <HistoryDrawer user={selectedHistoryUser} />}
                </Drawer>
              </CardContent>
            </Card>
          )
        })}
      </div>

      {usersWithStatuses.length === 0 && (
        <Card className="p-8 text-center">
          <p className={`text-muted-foreground ${tvMode ? "text-4xl" : "text-lg"}`}>No status updates yet</p>
        </Card>
      )}

      {/* Status Update Input */}
      {!tvMode && (
        <Card className="p-6 mt-6">
          <div className="flex flex-col gap-4">
            <h3 className="text-lg font-semibold">Update Status</h3>
            <div className="flex gap-4">
              <Input
                autoFocus
                type="text"
                placeholder="What's your status?"
                className="flex-1 text-base"
                value={statusInput}
                onChange={(e) => setStatusInput(e.target.value)}
                onKeyDown={(e) => {
                  if (e.key === "Enter" && !e.shiftKey) {
                    e.preventDefault()
                    updateStatus().catch((error) => {
                      toast.error(`Failed to update status: ${error as Error}`)
                    })
                  }
                }}
              />
              <SubmitButton
                onClick={updateStatus}
                disabled={updatingStatus}
                className="px-6"
              >
                {selectedUsers.length > 0 ? `Update ${selectedUsers.length} Users` : "Update Status"}
              </SubmitButton>
            </div>
            {selectedUsers.length > 0 && (
              <p className="text-sm text-muted-foreground">Updating status for {selectedUsers.length} selected users</p>
            )}
          </div>
        </Card>
      )}

      {/* Global Status History Drawer */}
      <div className="flex justify-center mt-6">
        <Drawer>
          <DrawerTrigger asChild>
            <Button variant="outline" className={tvMode ? "text-3xl p-6" : ""}>
              View All Status History
            </Button>
          </DrawerTrigger>
          <HistoryDrawer />
        </Drawer>
      </div>
    </div>
  )
}
//'use client';

//import { createClient } from '@/utils/supabase';
//import { useState, useEffect, useCallback, useRef } from 'react';
//import { useAuth, useTVMode } from '@/components/context';
//import {
  //getRecentUsersWithStatuses,
  //updateStatuses,
  //updateUserStatus,
  //type UserWithStatus,
//} from '@/lib/hooks';
//import {
  //BasedAvatar,
  //Drawer,
  //DrawerTrigger,
  //Loading,
//} from '@/components/ui';
//import { SubmitButton } from '@/components/default';
//import { toast } from 'sonner';
//import { HistoryDrawer } from '@/components/status';
//import type { Profile } from '@/utils/supabase';
//import type { RealtimeChannel } from '@supabase/supabase-js';
//import { makeConditionalClassName } from '@/lib/utils';

//export const StatusList = (initialStatuses: UserWithStatus[]) => {
  //const { isAuthenticated } = useAuth();
  //const { tvMode } = useTVMode();
  //const [loading, setLoading] = useState(true);
  //const [selectedUsers, setSelectedUsers] = useState<UserWithStatus[]>([]);
  //const [selectAll, setSelectAll] = useState(false);
  //const [updatingStatus, setUpdatingStatus] = useState(false);
  //const [statusInput, setStatusInput] = useState('');
  //const [usersWithStatuses, setUsersWithStatuses] = useState<UserWithStatus[]>(initialStatuses);
  //const [selectedHistoryUser, setSelectedHistoryUser] = useState<UserWithStatus | null>(null);
  //const channelRef = useRef<RealtimeChannel | null>(null);
  //const supabaseRef = useRef(createClient());

  //const fetchRecentUsersWithStatuses = useCallback(async () => {
    //const response = await getRecentUsersWithStatuses();
    //if (response.success) return response.data;
    //else toast.error('Error fetching users!');
    //return [];
  //}, []);

  //useEffect(() => {
    //const loadData = async () => {
      //const data = await fetchRecentUsersWithStatuses();
      //setUsersWithStatuses(data);
    //};
    //loadData()
      //.catch(()=> toast.error('Error fetching users.'))
      //.finally(() => setLoading(false));
  //}, [fetchRecentUsersWithStatuses, isAuthenticated]);

  //const updateStatus = useCallback(async () => {
    //try {
      //if (!isAuthenticated) throw new Error('Not authenticated.');
      //if (!statusInput.trim()) throw new Error('Not a valid status.')

      //setUpdatingStatus(true);
      //if (selectedUsers.length === 0) {
        //const result = await updateUserStatus(statusInput);
        //if (!result.success) throw new Error('Could not updateUserStatus!')
        //toast.success('Status updated!');
      //} else {
        //const result = await updateStatuses(selectedUsers, statusInput);
      //}

    //} catch (error) {
      //toast.error(`Error updating statuses: ${error as Error}`);
    //}
  //});

  //return (
    //<div/>
  //);
//};
